@@ -1213,6 +1213,96 @@ nsPipeTransport::ExecPrompt(const char* command, const char* prompt,
   return NS_OK;
 }
 
+// Read the next line
+NS_IMETHODIMP
+nsPipeTransport::ReadLine(PRInt32 maxOutputLen,
+                            char* *_retval)
+{
+  nsresult rv;
+
+  DEBUG_LOG(("nsPipeTransport::ReadLine: maxOutputLen=%d\n", maxOutputLen));
+
+  if (!_retval)
+    return NS_ERROR_NULL_POINTER;
+
+  if (!mInputStream) {
+    nsCOMPtr<nsIInputStream> inputStream;
+    rv = OpenInputStream(0, PRUint32(-1), 0, getter_AddRefs(inputStream));
+    if (NS_FAILED(rv)) return rv;
+  }
+
+  if (mStdoutStream != STREAM_SYNC_OPEN)
+    return NS_ERROR_NOT_AVAILABLE;
+
+
+  PRInt32 returnCount = -1;
+
+  if (maxOutputLen != 0) {
+    char buf[kCharMax];
+    PRUint32 readCount, readMax;
+
+    PRUint32 remainingCount = (maxOutputLen > 0) ? maxOutputLen : kCharMax;
+    
+    if (mExecBuf.Length()>0) {
+      mExecBuf.ReplaceSubstring("\r\n", "\n");
+      mExecBuf.ReplaceSubstring("\r", "\n");
+      returnCount = mExecBuf.Find("\n", PR_FALSE, 0);
+      DEBUG_LOG(("nsPipeTransport::ReadLine: returnCount=%d\n", returnCount));
+    }
+    
+    if (returnCount < 0) {
+      while (remainingCount > 0) {
+        readMax = (remainingCount < kCharMax) ? remainingCount : kCharMax;
+  
+        rv = mInputStream->Read((char *) buf, kCharMax, &readCount);
+        if (NS_FAILED(rv)) return rv;
+  
+        if (readCount < 0)
+          return NS_ERROR_FAILURE;
+  
+        if (readCount == 0)
+          break;             // End-of-file
+  
+        mExecBuf.Append(buf, readCount);
+  
+        if (mExecBuf.Length() >= 1) {
+          mExecBuf.ReplaceSubstring("\r\n", "\n");
+          mExecBuf.ReplaceSubstring("\r", "\n");
+          returnCount = mExecBuf.Find("\n", PR_FALSE, 0);
+  
+          if (returnCount >= 0) {
+            break;
+          }
+        }
+  
+        if (maxOutputLen > 0) {
+          // Limited read
+          remainingCount -= readCount;
+        } else {
+          // Unlimited read
+          remainingCount = kCharMax;
+        }
+      }
+    }
+    if (returnCount < 0)
+      returnCount = mExecBuf.Length();  // Return everything
+  }
+  
+  // Duplicate output string and return it
+  nsCAutoString outStr("");
+  if (returnCount >= 0) {
+    mExecBuf.Left(outStr, returnCount);
+    mExecBuf.Cut(0,returnCount+1);
+  }
+  *_retval = nsCRT::strdup(outStr.get());
+  if (!*_retval)
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  DEBUG_LOG(("nsPipeTransport::readLine: *_retval='%s'\n", *_retval));
+
+  return NS_OK;
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 // nsIRequest methods
 ///////////////////////////////////////////////////////////////////////////////
